"""client for policyApiServer"""

import logging
import time
import base64
import json
import httplib2

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

from . import ClientException

__all__ = ['PolicyApiServerException', 'validate', 'Client']


class PolicyApiServerException(ClientException):
    """Exception for PolicyApiServer client"""
    pass


def validate(request):
    """ Interface for using HTTP(S) requests
to the PolicyApiServer.  It can hold
onto a transaction id until a commit
or rollback."""
    if 'body' in request:
        try:
            result = json.loads(request['body'])
            logging.info(("Result: " + request['body']))
            return result
        except (IOError, ValueError):
            logging.info(request)
            logging.exception('API failure: could not parse JSON response')
            raise PolicyApiServerException('Bad JSON')
    # if no body was returned, give the status code instead
    return request['status_code']


# request type constants
GET = 'GET'
POST = 'POST'
PUT = 'PUT'


def try_api_request(func):
    """function decorator for logging that a client exception has been raised"""

    def func_wrapper(obj, data=None):
        """logs any exceptions that happen so the failed API call is easy to see"""
        try:
            return func(obj, data)
        except:
            logging.exception('Failure: client error while executing "' + func.__name__ + '"')
            raise

    return func_wrapper


class Client(object):
    """ makes http(s) requests to policy api server"""

    def _api_request(self, path, request_type, params=False, data=False):
        """ generic api request to server, can use http or https"""

        # construct request headers
        request_params = {'method': request_type}
        # append the base path for PolicyApiServer requets
        path = self._client_path + path
        if params:
            path = path + "?" + urlencode(params)
        if data:
            request_params['body'] = data
        request_params['url'] = path
        # add basic auth to request header if necessary
        if self._options['uses_authentication']:
            user_pass = [self._options['username'], self._options['password']]
            auth_encoded = base64.b64encode(str.encode(str(":".join(user_pass)))).decode()
            auth_header = 'Basic %s' % str(auth_encoded)
            request_params['headers'] = {'Authorization': auth_header}

        # make request with connection
        try:
            self.conn.request(**request_params)
            response = self.conn.getresponse()
            data = response.read().decode('utf-8')

            req = {'status_code': response.status, 'body': data}
            return validate(req)
        except:
            logging.info('API failure: server connection unsuccessful')
            raise PolicyApiServerException('Connection Error')

    @try_api_request
    def check_status(self, no_flush=False):
        """/status"""
        if no_flush:
            flush_param = {"noflush": True}
            return self._api_request('/status', GET, params=flush_param)
        else:
            return self._api_request('/status', GET)

    @try_api_request
    def start(self, needs_id):
        """/start"""
        result = self._api_request('/start', POST)
        if needs_id:
            self.transaction_id = result['Transaction ID']
        return result

    @try_api_request
    def commit(self, _):
        """/commit"""
        transaction = {'TransactionId': self.transaction_id}
        return self._api_request('/commit', POST, params=transaction)

    @try_api_request
    def rollback(self, _):
        """/rollback"""
        transaction = {'TransactionId': self.transaction_id}
        return self._api_request('/rollback', POST, params=transaction)

    @try_api_request
    def get_ips_and_urls(self, category):
        """/urls"""
        return self._api_request('/urls', GET, params=category)

    @try_api_request
    def add_urls(self, urls):
        """/urls"""
        urls['Transaction ID'] = self.transaction_id
        return self._api_request('/urls', POST, data=json.dumps(urls))

    @try_api_request
    def delete_urls(self, urls):
        """/urls"""
        urls['Transaction ID'] = self.transaction_id
        return self._api_request('/delete/urls', POST, data=json.dumps(urls))

    @try_api_request
    def add_categories(self, category):
        """/"""
        category['Transaction ID'] = self.transaction_id
        return self._api_request('', PUT, data=json.dumps(category))

    @try_api_request
    def delete_categories(self, category):
        """/"""
        category['Transaction ID'] = self.transaction_id
        return self._api_request('/delete', POST, data=json.dumps(category))

    @try_api_request
    def get_categories(self, _):
        """/"""
        transaction = {}
        if self.transaction_id is not None:
            transaction['Transaction ID'] = self.transaction_id
        return self._api_request('/all', GET, params=transaction)

    @try_api_request
    def get_api_categories(self, _):
        """/api"""
        transaction = {}
        if self.transaction_id is not None:
            transaction['Transaction ID'] = self.transaction_id
        return self._api_request('', GET, params=transaction)

    def wait_for_commit_enforcement(self):
        """calls status until it returns 'Done'"""
        data = self.check_status(True)
        num_tries = 0
        while data["Build Status"] == "In Progress" and num_tries < self.MAX_STATUS_CHECKS:
            num_tries += 1
            time.sleep(self.WAIT_BEFORE_STATUS_CHECKS)
            data = self.check_status(True)
        # raise exception if it ended due to timeout
        if num_tries == self.MAX_STATUS_CHECKS:
            raise PolicyApiServerException("Wait for commit transaction exceeded max wait time")

    def reset(self):
        """unsets any state that may be useful accross API calls"""
        self.transaction_id = None

    def setup_connection(self):
        """creates an http or https connection to be used for all api calls"""
        try:
            connection_params = {'host': self._options['server'], 'port': self._options['port']}
            if self._options['use_https']:
                # it's possible to use https and not verify certificate
                if self._options['verify_cert']:
                    connection_params['key_file'] = self._options['key']
                    connection_params['cert_file'] = self._options['cert']
                else:
                    connection_params['disable_ssl_certificate_validation'] = True
                self.conn = httplib2.HTTPSConnectionWithTimeout(**connection_params)
            else:
                self.conn = httplib2.HTTPConnectionWithTimeout(**connection_params)
        except:
            logging.info('API failure: server connection unsuccessful')
            raise PolicyApiServerException('Connection Error')

    def __del__(self):
        self.conn.close()

    def __init__(self, options):
        self.transaction_id = None  # accessed by update function externally
        self._client_path = "/api/web/v1/categories"
        self._options = options
        self.WAIT_BEFORE_STATUS_CHECKS = 2  # seconds
        self.MAX_STATUS_CHECKS = 10

        self.setup_connection()
