"""function to generate ips for the policy api server command line sample code"""

import random
import socket
import struct
import copy

__all__ = ['generate_ipv4s', 'generate_ipv6s', 'generate_ipv4_ranges', 'generate_ipv6_ranges']

DEFAULT_NUM = 1000  # number of ips/ip ranges generated in each file
DEFAULT_RANGE_SIZE = 128  # must be less than 2^32 for this example

IPV4 = socket.AF_INET
IPV6 = socket.AF_INET6


# utility functions
def pack(ip_val):
    """pack an value into byte form.
    returns a byte array"""
    return struct.pack('>I', ip_val)


def generate_u32(limit):
    """generate a random 32 bit number with room for a range"""
    return random.randint(1, 0xffffffff - limit)


def generate_u128(limit):
    """generate a 4 random 32 bit numbers with room for a range
    returns as a list of 4 uints"""
    return [generate_u32(0) for _ in range(3)] + [generate_u32(limit)]


def format_ipv6(values):
    """turns a list of 4 uints into a 128 byte array"""
    return ''.join([pack(val) for val in values])


# generate ipv6 range functions
def generate_ipv6_range(max_range_size):
    """generate a single ipv6 range"""
    ip_range_size = random.randint(1, max_range_size)
    ip_start = generate_u128(ip_range_size)
    ip_end = copy.copy(ip_start)
    ip_end[3] += ip_range_size
    return socket.inet_ntop(IPV6, format_ipv6(ip_start)) + '-' + socket.inet_ntop(IPV6, format_ipv6(ip_end))


def generate_ipv6_ranges(out_file, num_ip_ranges, max_range_size):
    """generate and output to file a list of ipv6 ranges"""
    ip_string = "\n".join([generate_ipv6_range(max_range_size) for _ in range(0, num_ip_ranges)])
    with open(out_file, 'w') as outfile:
        outfile.write(ip_string)


# generate ipv4 range functions
def generate_ipv4_range(max_range_size):
    """generate a single ipv4 range"""
    ip_range_size = random.randint(1, max_range_size)
    ip_val = generate_u32(ip_range_size)
    return '%s - %s' % (socket.inet_ntop(IPV4, pack(ip_val)), socket.inet_ntop(IPV4, pack(ip_val + ip_range_size)))


def generate_ipv4_ranges(out_file, num_ip_ranges, max_range_size):
    """generate and output to file a list of ipv4 ranges"""
    ip_string = "\n".join([generate_ipv4_range(max_range_size) for _ in range(0, num_ip_ranges)])
    with open(out_file, 'w') as outfile:
        outfile.write(ip_string)


# generate ipv6 functions
def generate_ipv6():
    """creates a single random ipv6 value"""
    return socket.inet_ntop(IPV6, format_ipv6(generate_u128(0)))


def generate_ipv6s(out_file, num_ips):
    """generate random ipv6s and output to file"""
    ip_string = "\n".join([generate_ipv6() for _ in range(0, num_ips)])
    with open(out_file, 'w') as outfile:
        outfile.write(ip_string)


# generate ipv4 functions
def generate_ipv4():
    """creates a single random ipv4 value"""
    return socket.inet_ntoa(pack(generate_u32(0)))


def generate_ipv4s(out_file, num_ips):
    """generate random ipv4s and output to file"""
    ip_string = "\n".join([generate_ipv4() for _ in range(0, num_ips)])
    with open(out_file, 'w') as outfile:
        outfile.write(ip_string)


if __name__ == "__main__":
    IP4_FILE = str(DEFAULT_NUM) + '_ip4.txt'
    generate_ipv4s(IP4_FILE, DEFAULT_NUM)

    IP6_FILE = str(DEFAULT_NUM) + '_ip6.txt'
    generate_ipv6s(IP6_FILE, DEFAULT_NUM)

    IP4_RANGE_FILE = str(DEFAULT_NUM) + '_ip4_ranges.txt'
    generate_ipv4_ranges(IP4_RANGE_FILE, DEFAULT_NUM, DEFAULT_RANGE_SIZE)

    IP6_RANGE_FILE = str(DEFAULT_NUM) + '_ip6_ranges.txt'
    generate_ipv6_ranges(IP6_RANGE_FILE, DEFAULT_NUM, DEFAULT_RANGE_SIZE)
