"""Sample usage of PolicyApiServer interface python code.
This example takes command line parameters to add (or remove)
URLs, IPs, and IP ranges to API categories."""

import logging
import getopt
import sys

from clients import policy_api_server_client
from clients import ClientException

__all__ = ["PARAMS", "POLICY_API_SERVER_PORT", "CLI"]

PARAMS = ["pw", "cat", "inputFile", "s", "p", "u", "d"]
POLICY_API_SERVER_PORT = 15873


class CLI(object):
    """
    PolicyApiServer CLI
    view README.txt for list of options and examples of usage
    """

    def wait_for_commit_enforcement(self, _):
        """For use after a commit. Polls the status every N seconds to see if
        the custom database has been created and loaded.  Useful for verifiying that newly added URLs/IPs
        are enforced."""
        try:
            self.client.wait_for_commit_enforcement()
        except ClientException as e:
            logging.exception("Error: %s" % e)

    def category_exists(self):
        """Get the api categories and check if one we are trying to add urls to already exists"""
        result = self.client.get_api_categories()
        api_categories = result['Categories']
        for api_category in api_categories:
            if api_category["Category Name"] == self.config['category']:
                return True
        return False

    def _run(self):
        """Adds urls to the api category"""
        # load the urls from file into an array
        urls = []
        if 'url_file' in self.config:
            with open(self.config['url_file'], "r") as url_file:
                urls = url_file.readlines()
                urls = [url.strip() for url in urls]  # remove carriage returns and whitespace
        # load the ips from file into an array
        ips = []
        if 'ip_file' in self.config:
            with open(self.config['ip_file'], "r") as ip_file:
                ips = ip_file.readlines()
                ips = [ip.strip() for ip in ips]  # remove carriage returns and whitespace

        # start a transaction and save id
        # NOTE: this bool param exists for testing purposes
        self.client.start(True)

        # if the category doesn't already exist, then add it
        if not self.category_exists():
            request_json = {"Categories": [{"Category Name": self.config['category']}]}
            self.client.add_categories(request_json)

        # add urls to category
        request_json = {
            'Category Name': self.config['category'],
            'URLs': urls,
            'IPs': ips
        }

        # add or remove the urls (depending on -d option)
        if self.config['isDelete']:
            self.client.delete_urls(request_json)
        else:
            self.client.add_urls(request_json)

        # commit the urls (and new category if necessary)
        self.client.commit()

    def _load_defaults(self):
        """get default values for policy api server client"""
        self.config = {
            'client':
            {
                'use_https': True,
                'verify_cert': False,
                'uses_authentication': True
            },
            'debug': True,
            'isDelete': False
        }

    def _get_options(self):
        """interpret command line options to set config"""

        try:
            args = sys.argv[1:]
            options = [arg for arg in args if '-' in arg]
            values = [arg for arg in args if '-' not in arg]
            for idx, option in enumerate(options):
                if 's' in option:
                    self.config['client']['server'] = values[idx]
                elif 'pw' in option:
                    self.config['client']['password'] = values[idx]
                    continue
                elif 'cat' in option:
                    self.config['category'] = values[idx]
                elif 'inputUrlFile' in option:
                    self.config['url_file'] = values[idx]
                    continue
                elif 'inputIpFile' in option:
                    self.config['ip_file'] = values[idx]
                    continue
                elif 'p' in option:
                    self.config['client']['port'] = values[idx]
                elif 'u' in option:
                    self.config['client']['username'] = values[idx]
                elif 'd' in option:
                    self.config['isDelete'] = True
        except (getopt.GetoptError, IndexError):
            pass

        # cover required parameters
        if 'category' not in self.config:
            raise ValueError("Custom category name required")

        if 'username' not in self.config['client'] or 'password' not in self.config['client']:
            raise ValueError("Username or password cannot be blank")

        if 'url_file' not in self.config and 'ip_file' not in self.config:
            raise ValueError("URL or IP file required")

        # log optional parameters that are recommended
        if 'server' not in self.config['client']:
            self.config['client']['server'] = 'localhost'
            print("No server specified- defaulting to 'localhost'")
        if 'port' not in self.config['client']:
            self.config['client']['port'] = POLICY_API_SERVER_PORT
            print("No port specified- defaulting to " + str(POLICY_API_SERVER_PORT))

    def __init__(self):
        # setup the client
        self._load_defaults()
        self._get_options()
        self.client = policy_api_server_client.Client(self.config['client'])

        # default to showing server responses
        if 'debug' in self.config:
            logging.basicConfig(level=logging.INFO)

        self._run()

if __name__ == "__main__":
    CLI()
